@@ -5,8 +5,9 @@ use warnings;
 use integer;
 use bytes;
 
-use MIME::Base64 qw<decode_base64>;
 use Carp         qw<croak>;
+use MIME::Base64 qw<decode_base64>;
+use POSIX        qw<strftime>;
 
 use GDPR::IAB::TCFv2::BitField;
 use GDPR::IAB::TCFv2::BitUtils qw<is_set
@@ -20,14 +21,40 @@ use GDPR::IAB::TCFv2::BitUtils qw<is_set
 use GDPR::IAB::TCFv2::PublisherRestrictions;
 use GDPR::IAB::TCFv2::RangeSection;
 
-our $VERSION = "0.07";
+our $VERSION = "0.08";
 
-use constant CONSENT_STRING_TCF2_SEPARATOR => '.';
-use constant CONSENT_STRING_TCF2_PREFIX    => 'C';
-use constant MIN_BYTE_SIZE                 => 29;
-use constant MIN_BIT_SIZE                  => 8 * MIN_BYTE_SIZE;
-use constant TCF_VERSION                   => 2;
-use constant ASSUMED_MAX_VENDOR_ID         => 0x7FFF;   # 32767 or (1 << 15) -1
+use constant {
+    CONSENT_STRING_TCF2_SEPARATOR => '.',
+    CONSENT_STRING_TCF2_PREFIX    => 'C',
+    MIN_BYTE_SIZE                 => 29,
+    TCF_VERSION                   => 2,
+    ASSUMED_MAX_VENDOR_ID         => 0x7FFF,    # 32767 or (1 << 15) -1
+    MAX_SPECIAL_FEATURE_ID        => 12,
+    MAX_PURPOSE_ID                => 24,
+    DATE_FORMAT_ISO_8601          => '%Y-%m-%dT%H:%M:%SZ',
+
+    VERSION_OFFSET                       => 0,
+    CREATED_OFFSET                       => 6,
+    LAST_UPDATED_OFFSET                  => 42,
+    CMP_ID_OFFSET                        => 78,
+    CMP_VERSION_OFFSET                   => 90,
+    CONSENT_SCREEN_OFFSET                => 102,
+    CONSENT_LANGUAGE_OFFSET              => 108,
+    VENDOR_LIST_VERSION_OFFSET           => 120,
+    POLICY_VERSION_OFFSET                => 132,
+    SERVICE_SPECIFIC_OFFSET              => 138,
+    USE_NON_STANDARD_STACKS_OFFSET       => 139,
+    SPECIAL_FEATURE_OPT_IN_OFFSET        => 140,
+    PURPOSE_CONSENT_ALLOWED_OFFSET       => 152,
+    PURPOSE_LIT_ALLOWED_OFFSET           => 176,
+    PURPOSE_ONE_TREATMENT_OFFSET         => 200,
+    PUBLISHER_COUNTRY_CODE_OFFSET        => 201,
+    MAX_VENDOR_ID_CONSENT_OFFSET         => 213,
+    VENDOR_CONSENT_RANGE_ENCODING_OFFSET => 229,
+    VENDOR_CONSENT_OFFSET                => 230,
+};
+
+use overload q<""> => \&tc_string;
 
 INIT {
     if ( my $native_decode_base64url = MIME::Base64->can("decode_base64url") )
@@ -36,12 +63,28 @@ INIT {
 
         *_decode_base64url = $native_decode_base64url;
     }
+
+    eval {
+        require JSON;
+
+        if ( my $native_json_true = JSON->can("true") ) {
+            no warnings q<redefine>;
+
+            *_json_true = $native_json_true;
+        }
+
+        if ( my $native_json_false = JSON->can("false") ) {
+            no warnings q<redefine>;
+
+            *_json_false = $native_json_false;
+        }
+    };
 }
 
 # ABSTRACT: gdpr iab tcf v2 consent string parser
 
 sub Parse {
-    my ( $klass, $tc_string ) = @_;
+    my ( $klass, $tc_string, %opts ) = @_;
 
     croak 'missing gdpr consent string' unless $tc_string;
 
@@ -51,10 +94,18 @@ sub Parse {
     my $data_size = length($data);
 
     croak "vendor consent strings are at least @{[ MIN_BYTE_SIZE ]} bytes long"
-      if $data_size < MIN_BIT_SIZE;
+      if $data_size < 8 * MIN_BYTE_SIZE;
+
+    my %options = (
+        json => $opts{json} || {},
+    );
+
+    $options{json}->{date_format}    ||= DATE_FORMAT_ISO_8601;
+    $options{json}->{boolean_values} ||= [ _json_false(), _json_true() ];
 
     my $self = {
         data                           => $data,
+        options                        => \%options,
         tc_string                      => $tc_string,
         vendor_consents                => undef,
         legitimate_interest_max_vendor => undef,
@@ -64,10 +115,10 @@ sub Parse {
 
     bless $self, $klass;
 
-    croak 'consent string is not tcf version 2'
+    croak "consent string is not tcf version @{[ TCF_VERSION ]}"
       unless $self->version == TCF_VERSION;
 
-    croak 'invalid vendor list version' unless $self->vendor_list_version;
+    croak 'invalid vendor list version' if $self->vendor_list_version == 0;
 
     # parse consent
 
@@ -89,16 +140,22 @@ sub Parse {
     return $self;
 }
 
+sub tc_string {
+    my $self = shift;
+
+    return $self->{tc_string};
+}
+
 sub version {
     my $self = shift;
 
-    return get_uint6( $self->{data}, 0 );
+    return get_uint6( $self->{data}, VERSION_OFFSET );
 }
 
 sub created {
     my $self = shift;
 
-    my ( $seconds, $nanoseconds ) = $self->_get_epoch(6);
+    my ( $seconds, $nanoseconds ) = $self->_get_epoch(CREATED_OFFSET);
 
     return wantarray ? ( $seconds, $nanoseconds ) : $seconds;
 }
@@ -106,7 +163,7 @@ sub created {
 sub last_updated {
     my $self = shift;
 
-    my ( $seconds, $nanoseconds ) = $self->_get_epoch(42);
+    my ( $seconds, $nanoseconds ) = $self->_get_epoch(LAST_UPDATED_OFFSET);
 
     return wantarray ? ( $seconds, $nanoseconds ) : $seconds;
 }
@@ -125,49 +182,49 @@ sub _get_epoch {
 sub cmp_id {
     my $self = shift;
 
-    return get_uint12( $self->{data}, 78 );
+    return get_uint12( $self->{data}, CMP_ID_OFFSET );
 }
 
 sub cmp_version {
     my $self = shift;
 
-    return get_uint12( $self->{data}, 90 );
+    return get_uint12( $self->{data}, CMP_VERSION_OFFSET );
 }
 
 sub consent_screen {
     my $self = shift;
 
-    return get_uint6( $self->{data}, 102 );
+    return get_uint6( $self->{data}, CONSENT_SCREEN_OFFSET );
 }
 
 sub consent_language {
     my $self = shift;
 
-    return get_char6_pair( $self->{data}, 108 );
+    return get_char6_pair( $self->{data}, CONSENT_LANGUAGE_OFFSET );
 }
 
 sub vendor_list_version {
     my $self = shift;
 
-    return get_uint12( $self->{data}, 120 );
+    return get_uint12( $self->{data}, VENDOR_LIST_VERSION_OFFSET );
 }
 
 sub policy_version {
     my $self = shift;
 
-    return get_uint6( $self->{data}, 132 );
+    return get_uint6( $self->{data}, POLICY_VERSION_OFFSET );
 }
 
 sub is_service_specific {
     my $self = shift;
 
-    return is_set( $self->{data}, 138 );
+    return is_set( $self->{data}, SERVICE_SPECIFIC_OFFSET );
 }
 
 sub use_non_standard_stacks {
     my $self = shift;
 
-    return is_set( $self->{data}, 139 );
+    return is_set( $self->{data}, USE_NON_STANDARD_STACKS_OFFSET );
 }
 
 sub is_special_feature_opt_in {
@@ -176,7 +233,7 @@ sub is_special_feature_opt_in {
     croak "invalid special feature id $id: must be between 1 and 12"
       if $id < 1 || $id > 12;
 
-    return is_set( $self->{data}, 140 + $id - 1 );
+    return is_set( $self->{data}, SPECIAL_FEATURE_OPT_IN_OFFSET + $id - 1 );
 }
 
 sub is_purpose_consent_allowed {
@@ -185,7 +242,7 @@ sub is_purpose_consent_allowed {
     croak "invalid purpose id $id: must be between 1 and 24"
       if $id < 1 || $id > 24;
 
-    return is_set( $self->{data}, 152 + $id - 1 );
+    return is_set( $self->{data}, PURPOSE_CONSENT_ALLOWED_OFFSET + $id - 1 );
 }
 
 sub is_purpose_legitimate_interest_allowed {
@@ -194,25 +251,25 @@ sub is_purpose_legitimate_interest_allowed {
     croak "invalid purpose id $id: must be between 1 and 24"
       if $id < 1 || $id > 24;
 
-    return is_set( $self->{data}, 176 + $id - 1 );
+    return is_set( $self->{data}, PURPOSE_LIT_ALLOWED_OFFSET + $id - 1 );
 }
 
 sub purpose_one_treatment {
     my $self = shift;
 
-    return is_set( $self->{data}, 200 );
+    return is_set( $self->{data}, PURPOSE_ONE_TREATMENT_OFFSET );
 }
 
 sub publisher_country_code {
     my $self = shift;
 
-    return get_char6_pair( $self->{data}, 201 );
+    return get_char6_pair( $self->{data}, PUBLISHER_COUNTRY_CODE_OFFSET );
 }
 
 sub max_vendor_id_consent {
     my $self = shift;
 
-    return get_uint16( $self->{data}, 213 );
+    return get_uint16( $self->{data}, MAX_VENDOR_ID_CONSENT_OFFSET );
 }
 
 sub max_vendor_id_legitimate_interest {
@@ -240,6 +297,96 @@ sub check_publisher_restriction {
       ->check_publisher_restriction( $purpose_id, $restrict_type, $vendor );
 }
 
+sub _format_date {
+    my ( $self, $epoch, $nanoseconds ) = @_;
+
+    return $epoch if !!$self->{options}->{json}->{use_epoch};
+
+    my $format = $self->{options}->{json}->{date_format};
+
+    return $format->( $epoch, $nanoseconds ) if ref($format) eq ref( sub { } );
+
+    return strftime( $format, gmtime($epoch) );
+}
+
+sub _json_true { 1 == 1 }
+
+sub _json_false { 1 == 0 }
+
+sub _format_json_subsection {
+    my ( $self, @data ) = @_;
+
+    if ( !!$self->{options}->{json}->{compact} ) {
+        return [ map { $_->[0] } grep { $_->[1] } @data ];
+    }
+
+    my $verbose = !!$self->{options}->{json}->{verbose};
+
+    return { map { @{$_} } grep { $verbose || $_->[1] } @data };
+}
+
+sub TO_JSON {
+    my $self = shift;
+
+    my ( $false, $true ) = @{ $self->{options}->{json}->{boolean_values} };
+
+    my $created      = $self->_format_date( $self->created );
+    my $last_updated = $self->_format_date( $self->last_updated );
+
+    return {
+        tc_string               => $self->tc_string,
+        version                 => $self->version,
+        created                 => $created,
+        last_updated            => $last_updated,
+        cmp_id                  => $self->cmp_id,
+        cmp_version             => $self->cmp_version,
+        consent_screen          => $self->consent_screen,
+        consent_language        => $self->consent_language,
+        vendor_list_version     => $self->vendor_list_version,
+        policy_version          => $self->policy_version,
+        is_service_specific     => $self->is_service_specific ? $true : $false,
+        use_non_standard_stacks => $self->use_non_standard_stacks
+        ? $true
+        : $false,
+        purpose_one_treatment => $self->purpose_one_treatment ? $true : $false,
+        publisher_country_code  => $self->publisher_country_code,
+        special_features_opt_in => $self->_format_json_subsection(
+            map {
+                [ $_ => $self->is_special_feature_opt_in($_) ? $true : $false ]
+            } 1 .. MAX_SPECIAL_FEATURE_ID
+        ),
+        purposes_consent => $self->_format_json_subsection(
+            map {
+                [     $_ => $self->is_purpose_consent_allowed($_)
+                    ? $true
+                    : $false
+                ]
+            } 1 .. MAX_PURPOSE_ID,
+        ),
+        purposes_legitimate_interest => $self->_format_json_subsection(
+            map {
+                [     $_ => $self->is_purpose_legitimate_interest_allowed($_)
+                    ? $true
+                    : $false
+                ]
+            } 1 .. MAX_PURPOSE_ID,
+        ),
+        vendor_consents => $self->_format_json_subsection(
+            map { [ $_ => $self->vendor_consent($_) ? $true : $false ] }
+              1 .. $self->max_vendor_id_consent,
+        ),
+        vendor_legitimate_interests => $self->_format_json_subsection(
+            map {
+                [     $_ => $self->vendor_legitimate_interest($_)
+                    ? $true
+                    : $false
+                ]
+            } 1 .. $self->max_vendor_id_legitimate_interest,
+        ),
+    };
+}
+
+
 sub _parse_vendor_consents {
     my $self = shift;
 
@@ -247,11 +394,17 @@ sub _parse_vendor_consents {
 
     if ( $self->_is_vendor_consent_range_encoding ) {
         ( $vendor_consents, $legitimate_interest_start ) =
-          $self->_parse_range_section( $self->max_vendor_id_consent, 230 );
+          $self->_parse_range_section(
+            $self->max_vendor_id_consent,
+            VENDOR_CONSENT_OFFSET
+          );
     }
     else {
         ( $vendor_consents, $legitimate_interest_start ) =
-          $self->_parse_bitfield( $self->max_vendor_id_consent, 230 );
+          $self->_parse_bitfield(
+            $self->max_vendor_id_consent,
+            VENDOR_CONSENT_OFFSET
+          );
     }
 
     $self->{vendor_consents} = $vendor_consents;
@@ -352,9 +505,9 @@ sub _validate_and_decode_base64 {
         ^
         (?: [A-Za-z0-9-_]{4} )*
         (?:
-            [A-Za-z0-9-_]{2} [AEIMQUYcgkosw048] =?
+            [A-Za-z0-9-_]{2} [AEIMQUYcgkosw048]
         |
-            [A-Za-z0-9-_] [AQgw] (?:==)?
+            [A-Za-z0-9-_] [AQgw]
         )?
         \z
     }x;
@@ -372,7 +525,7 @@ sub _decode_base64url {
 sub _is_vendor_consent_range_encoding {
     my $self = shift;
 
-    return is_set( $self->{data}, 229 );
+    return is_set( $self->{data}, VENDOR_CONSENT_RANGE_ENCODING_OFFSET );
 }
 
 sub _parse_range_section {
@@ -416,15 +569,21 @@ __END__
 
 =for html <a href="https://cpants.cpanauthors.org/dist/GDPR-IAB-TCFv2"><img src="https://cpants.cpanauthors.org/dist/GDPR-IAB-TCFv2.svg" alt='Kwalitee'/></a>
 
-=for html <a href="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/linux.yml"><img src="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/linux.yml/badge.svg" alt='tests"/></a>
+=for html <a href="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/linux.yml"><img src="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/linux.yml/badge.svg" alt='tests'/></a>
 
-=for html <a href='https://ci.appveyor.com/project/peczenyj/gdpr-iab-tcfv2'><img src='https://ci.appveyor.com/api/projects/status/p2e2478cufcdqanl?svg=true' alt='appveyor'/></a>
+=for html <a href="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/windows.yml"><img src="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/windows.yml/badge.svg" alt='tests'/></a>
 
-=for html <a href='https://coveralls.io/github/peczenyj/GDPR-IAB-TCFv2?branch=main'><img src='https://coveralls.io/repos/github/peczenyj/GDPR-IAB-TCFv2/badge.svg?branch=main' alt='Coverage Status' /></a>
+=for html <a href="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/macos.yml"><img src="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/macos.yml/badge.svg" alt='tests'/></a>
+
+=for html <a href="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/perltidy.yml"><img src="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/perltidy.yml/badge.svg" alt='tests'/></a>
+
+=for html <a href="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/perlcritic.yml"><img src="https://github.com/peczenyj/GDPR-IAB-TCFv2/actions/workflows/perlcritic.yml/badge.svg" alt='tests'/></a>
+
+=for html <a href="https://coveralls.io/github/peczenyj/GDPR-IAB-TCFv2?branch=main"><img src="https://coveralls.io/repos/github/peczenyj/GDPR-IAB-TCFv2/badge.svg?branch=main" alt='Coverage Status' /></a>
 
 =for html <a href="https://github.com/peczenyj/GDPR-IAB-TCFv2/blob/master/LICENSE"><img src="https://img.shields.io/cpan/l/GDPR-IAB-TCFv2.svg" alt='license'/></a>
 
-=for html <a href="https://metacpan.org/dist/GDPR-IAB-TCFv2"><img src="https://img.shields.io/cpan/v/GDPR-IAB-TCFv2.svg" alt='cpan'/></A>
+=for html <a href="https://metacpan.org/dist/GDPR-IAB-TCFv2"><img src="https://img.shields.io/cpan/v/GDPR-IAB-TCFv2.svg" alt='cpan'/></a>
 
 =head1 NAME
 
@@ -432,7 +591,7 @@ GDPR::IAB::TCFv2 - Transparency & Consent String version 2 parser
 
 =head1 VERSION
 
-Version 0.07
+Version 0.08
 
 =head1 SYNOPSIS
 
@@ -506,8 +665,63 @@ Will die if can't decode the string.
         'CLcVDxRMWfGmWAVAHCENAXCkAKDAADnAABRgA5mdfCKZuYJez-NQm0TBMYA4oCAAGQYIAAAAAAEAIAEgAA.argAC0gAAAAAAAAAAAA'
     );
 
+    or
+
+    use GDPR::IAB::TCFv2;
+
+    my $consent = GDPR::IAB::TCFv2->Parse(
+        'CLcVDxRMWfGmWAVAHCENAXCkAKDAADnAABRgA5mdfCKZuYJez-NQm0TBMYA4oCAAGQYIAAAAAAEAIAEgAA.argAC0gAAAAAAAAAAAA',
+        json => {
+            verbose        => 0,
+            compact        => 1,
+            use_epoch      => 0,
+            boolean_values => [ 0, 1 ],
+            date_format    => '%Y%m%d',    # yyymmdd
+        },
+    );
+
+Parse may receive an optional hash parameter C<json> with the following properties:
+
+=over
+
+=item *
+
+C<verbose> changes the json encoding. By default we omit some false values such as C<vendor_consents> to create 
+a compact json representation. With C<verbose> we will present everything. See L<TO_JSON> for more details.
+
+=item *
+
+C<compact> changes the json encoding. All fields that are a mapping of something to a boolean will be changed to an array
+of all elements keys where the value is true. This affects the following fields:  C<special_features_opt_in>,
+C<purposes_consent>, C<purposes_legitimate_interest>, C<vendor_consents> and C<vendor_legitimate_interests>. See L<TO_JSON> for more details.
+
+=item *
+
+C<use_epoch> changes the json encode. By default we format the C<created> and C<last_updated> are converted to string using 
+L<ISO_8601|https://en.wikipedia.org/wiki/ISO_8601>. With C<use_epoch> we will return the unix epoch in seconds.
+See L<TO_JSON> for more details.
+
+=item *
+
+C<boolean_values> if present, expects an arrayref if two elements: the C<false> and the C<true> values to be used in json encoding.
+If omit, we will try to use C<JSON::false> and C<JSON::true> if the package L<JSON> is available, else we will fallback to C<0> and C<1>.
+
+=item *
+
+C<date_format> if present accepts two kinds of value: an C<string> (to be used on C<POSIX::strftime>) or a code reference to a subroutine that
+will be called with two arguments: epoch in seconds and nanoseconds. If omitted the format L<ISO_8601|https://en.wikipedia.org/wiki/ISO_8601> will be used
+except if the option C<use_epoch> is true.
+
+=back
+
 =head1 METHODS
 
+=head2 tc_string
+
+Returns the original consent string.
+
+The consent object L<GDPR::IAB::TCFv2> will call this method on string interpolations.
+
 =head2 version
 
 Version number of the encoding format. The value is 2 for this format.
@@ -673,7 +887,9 @@ Publisher restrictions are custom requirements specified by a publisher. In orde
 =over
 
 =item 1
-Vendors must always respect a restriction signal that disallows them the processing for a specific purpose regardless of whether or not they have declared that purpose to be “flexible”.
+
+Vendors must always respect a restriction signal that disallows them the processing for a specific purpose regardless of whether or not they have declared that purpose to be "flexible".
+
 =item 2
 
 Vendors that declared a purpose with a default legal basis (consent or legitimate interest respectively) but also declared this purpose as flexible must respect a legal basis restriction if present. That means for example in case they declared a purpose as legitimate interest but also declared that purpose as flexible and there is a legal basis restriction to require consent, they must then check for the consent signal and must not apply the legitimate interest signal.
@@ -684,6 +900,77 @@ For the avoidance of doubt:
 
 In case a vendor has declared flexibility for a purpose and there is no legal basis restriction signal it must always apply the default legal basis under which the purpose was registered aside from being registered as flexible. That means if a vendor declared a purpose as legitimate interest and also declared that purpose as flexible it may not apply a "consent" signal without a legal basis restriction signal to require consent.
 
+=head2 TO_JSON
+
+Will serialize the consent object into a hash reference. The objective is to be used by L<JSON> package.
+
+With option C<convert_blessed>, the encoder will call this method.
+
+    use strict;
+    use warnings;
+    use feature qw<say>;
+
+    use JSON;
+    use DateTime;
+    use DateTimeX::TO_JSON formatter => 'DateTime::Format::RFC3339';
+    use GDPR::IAB::TCFv2;
+
+    my $consent = GDPR::IAB::TCFv2->Parse(
+        'COyiILmOyiILmADACHENAPCAAAAAAAAAAAAAE5QBgALgAqgD8AQACSwEygJyAAAAAA',
+        json => {
+            compact     => 1,
+            date_format => sub { # can be omitted, with DateTimeX::TO_JSON
+                my ( $epoch, $ns ) = @_;
+
+                return DateTime->from_epoch( epoch => $epoch )
+                ->set_nanosecond($ns);
+            },
+        },
+    );
+
+    my $json    = JSON->new->convert_blessed;
+    my $encoded = $json->pretty->encode($consent);
+
+    say $encoded;
+
+    # outputs:
+
+    {
+    "tc_string":"COyiILmOyiILmADACHENAPCAAAAAAAAAAAAAE5QBgALgAqgD8AQACSwEygJyAAAAAA",
+    "consent_language":"EN",
+    "purposes_consent":[],
+    "vendor_legitimate_interests":[],
+    "cmp_id":3,
+    "purpose_one_treatment":false,
+    "special_features_opt_in":[],
+    "last_updated":"2020-04-27T20:27:54.200000000Z",
+    "use_non_standard_stacks":false,
+    "policy_version":2,
+    "version":2,
+    "vendor_consents":[
+        23,
+        42,
+        126,
+        127,
+        128,
+        587,
+        613,
+        626
+    ],
+    "is_service_specific":false,
+    "created":"2020-04-27T20:27:54.200000000Z",
+    "consent_screen":7,
+    "vendor_list_version":15,
+    "cmp_version":2,
+    "purposes_legitimate_interest":[],
+    "publisher_country_code":"AA"
+    }
+
+If L<JSON> is installed, the C<TO_JSON> method will use C<JSON::true> and C<JSON::false> as boolean value.
+
+By default it returns a compacted format where we omit the C<false> on fields like C<vendor_consents> and we convert the dates 
+using L<ISO_8601|https://en.wikipedia.org/wiki/ISO_8601>. This behaviour can be changed by extra option in the L<Parse> constructor.
+
 =head1 FUNCTIONS
 
 =head2 looksLikeIsConsentVersion2
