@@ -5,6 +5,8 @@ use integer;
 use bytes;
 use Math::BigInt;
 
+use Carp qw(croak confess);
+
 require Exporter;
 use base qw<Exporter>;
 
@@ -30,7 +32,7 @@ our @EXPORT_OK = qw<is_set
 sub is_set {
     my ( $data, $offset ) = @_;
 
-    # TODO check if offset is in range of $data
+    croak "index our of bounds on offset $offset" if $offset >= length($data);
 
     return substr( $data, $offset, 1 ) == 1;
 }
@@ -135,6 +137,14 @@ Will return true if the bit present on bit offset is 1.
 
     my $is_service_specific = is_set( $data, 138 );
 
+=head2 get_uint2
+
+Receive two parameters: data and bit offset.
+
+Will fetch 2 bits from data since bit offset and convert it an unsigned int.
+
+    my $value = get_uint2( $data, $offset );
+
 =head2 get_uint6
 
 Receive two parameters: data and bit offset.
@@ -151,7 +161,7 @@ Similar to L<GDPR::IAB::TCFv2::BitUtils::get_uint6> but perform increment the va
 
 Receives the data, bit offset and sequence size n.
 
-Returns a string of size n by concantenating L<GDPR::IAB::TCFv2::BitUtils::get_char6> calls.
+Returns a string of size n by concatenate L<GDPR::IAB::TCFv2::BitUtils::get_char6> calls.
 
     my $consent_language = get_char6_pair($data, 108, 2) # returns two letter country encoded as ISO_639-1 
 
